﻿using System.Diagnostics;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using HesabeDotNetCore.Models;
using Newtonsoft.Json;

namespace HesabeDotNetCore.Controllers
{
    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;

        public HomeController(ILogger<HomeController> logger)
        {
            _logger = logger;
        }

        public IActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// Action processes the checkout data. Captures users input, encryotes it, POSTs to the Checkout API and then redirects for payment
        /// </summary>
        /// <param name="paymentData"></param>
        /// <returns></returns>
        [HttpPost]        
        public async Task<IActionResult> Index(HesabeCheckoutData paymentData)
        {
            // Fill the payment data class object with the configuration values from the merchant profile
            paymentData.MerchantCode = HesabeConstants.MERCHANT_CODE;
            paymentData.FailureUrl = HesabeConstants.FAILURE_URL;
            paymentData.ResponseUrl = HesabeConstants.RESPONSE_URL;
            paymentData.Version = HesabeConstants.VERSION;

            // Initialize the HesabeCrypt encryption/decryption library using the KEY and IV from the configuration
            var hesabeCrypt = new HesabeCrypt(HesabeConstants.MERCHANT_KEY, HesabeConstants.MERCHANT_IV);

            // Serialize the payment data into JSON string
            var paymentDataJsonSerialized = JsonConvert.SerializeObject(paymentData);

            // Encrypt the JSON Serialized string
            var encryptedPaymentData = hesabeCrypt.Encrypt(paymentDataJsonSerialized);

            // POST the serialized string to the checkout API and recieve back the reponse
            var checkoutApiEncryptedResponse = await HesabeApiHelper.PostCheckoutApi(encryptedPaymentData);

            // Decrypt the response from the checkout API
            var checkoutApiDecryptedResponse = hesabeCrypt.Decrypt(checkoutApiEncryptedResponse);

            // Deserialize the JSON string into an object
            var checkoutApiResponse = JsonConvert.DeserializeObject<HesabeCheckoutResponse>(checkoutApiDecryptedResponse);

            // Redirect the user to the payment page using the token from the checkout API response
            return Redirect($"{HesabeConstants.PAYMENT_URL}?data={checkoutApiResponse.Response.Data}");
        }


        [HttpGet("PaymentFailure")]
        public IActionResult PaymentFailure(string data)
        {
            // Initialize the HesabeCrypt encryption/decryption library using the KEY and IV from the configuration
            var hesabeCrypt = new HesabeCrypt(HesabeConstants.MERCHANT_KEY, HesabeConstants.MERCHANT_IV);

            //Decrypt the response recieved in the data query string 
            var dataDecrypted = hesabeCrypt.Decrypt(data);

            //Trim the junk characters from the derypted response if any.
            dataDecrypted = dataDecrypted.Substring(0, dataDecrypted.LastIndexOf("}") + 1);

            //Desrialize the decrypted response
            var paymentResponse = JsonConvert.DeserializeObject<HesabePaymentResponse>(dataDecrypted);

            return View(paymentResponse);
        }

        [HttpGet("PaymentSuccess")]
        public IActionResult PaymentSuccess(string data)
        {
            // Initialize the HesabeCrypt encryption/decryption library using the KEY and IV from the configuration
            var hesabeCrypt = new HesabeCrypt(HesabeConstants.MERCHANT_KEY, HesabeConstants.MERCHANT_IV);

            //Decrypt the response recieved in the data query string 
            var dataDecrypted = hesabeCrypt.Decrypt(data);

            //Trim the junk characters from the derypted response if any.
            dataDecrypted = dataDecrypted.Substring(0, dataDecrypted.LastIndexOf("}") + 1);

            //Desrialize the decrypted response
            var paymentResponse = JsonConvert.DeserializeObject<HesabePaymentResponse>(dataDecrypted);

            return View(paymentResponse);
        }


        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}
